@@ -1,3 +1,5 @@
+import trans
+
 """Utilities for determing name variants.
 
    Copyright (c) Andrew Flegg <andrew@bleb.org> 2009.
@@ -32,6 +34,7 @@ def variants(name):
     if (name is None):
       return result
     
+    name = unicode(name).encode('trans')
     result.add(name)
     bits = name.split(' ')
     for bit in bits:
